@@ -7,6 +7,7 @@ from readerwriterlock import rwlock
 from confluent_kafka import KafkaError
 from typing import Callable
 from k2eg.broker import Broker
+from concurrent.futures import ThreadPoolExecutor
 
 _protocol_regex = r"^(pva?|ca)://((?:[A-Za-z0-9-_:]+(?:\.[A-Za-z0-9-_]+)*))$"
 
@@ -78,6 +79,9 @@ class dml:
         else:
             msg_id = list(decodec_msg.keys())[0]
             converted_msg = decodec_msg
+        
+        # Add message-size key with the received msgpack size
+        converted_msg['message-size'] = len(m_msg)
         return msg_id, converted_msg
 
     def __from_msgpack_compack(self, mc_msg):
@@ -111,7 +115,10 @@ class dml:
                 break   
         return msg_id, converted_msg
 
-
+    def process_event(self, topic_name, msg_id, decoded_message):
+        logging.debug(f"received event on topic {topic_name}")
+        self.__monitor_pv_handler[msg_id](msg_id, decoded_message)
+    
     def __consumer_handler(self):
         """ Consume message form kafka consumer
         after the message has been consumed the header 'k2eg-ser-type' is checked 
@@ -120,38 +127,40 @@ class dml:
             msgpack, 
             msgpack-compact
         """
-        while self.__consume_data:
-        #for msg in self.__consumer:
-            message = self.__broker.get_next_message()
-            if message is None: 
-                continue
-            if message.error():
-                if message.error().code() == KafkaError._PARTITION_EOF:
-                    # End of partition event
-                    logging.error(
-                        f"{message.topic()} [{message.partition()}]reached "+
-                        f"end at offset {message.offset()}"
-                    )
+        with  ThreadPoolExecutor(max_workers=10) as executor:
+            while self.__consume_data:
+                message = self.__broker.get_next_message()
+                if message is None: 
+                    continue
+                if message.error():
+                    if message.error().code() == KafkaError._PARTITION_EOF:
+                        # End of partition event
+                        logging.error(
+                            f"{message.topic()} [{message.partition()}]reached "+
+                            f"end at offset {message.offset()}"
+                        )
+                    else:
+                        continue
                 else:
-                    continue
-            else:
-                was_a_reply = False
-                #msg_id could be a reply id or pv name
-                msg_id, decoded_message = self.__decode_message(message)
-                if msg_id is None or decoded_message is None:
-                    continue
-                with self.reply_wait_condition:
-                    was_a_reply = msg_id in self.reply_message
-                    if was_a_reply is True:
-                        logging.debug(f"received reply on topic {message.topic()}")
-                        self.reply_message[msg_id] = decoded_message
-                        self.reply_wait_condition.notifyAll()
-                    elif msg_id in self.__monitor_pv_handler:
-                        logging.debug(f"received event on topic {message.topic()}")
-                        self.__monitor_pv_handler[msg_id](
-                            msg_id, decoded_message[msg_id]
-                            )
-                self.__broker.commit_current_fetched_message()
+                    was_a_reply = False
+                    #msg_id could be a reply id or pv name
+                    msg_id, decoded_message = self.__decode_message(message)
+                    if msg_id is None or decoded_message is None:
+                        continue
+                    with self.reply_wait_condition:
+                        was_a_reply = msg_id in self.reply_message
+                        if was_a_reply is True:
+                            # print(f"message received from topic: {message.topic()} offset: {message.offset()}")
+                            logging.debug(f"received reply on topic {message.topic()}")
+                            self.reply_message[msg_id] = decoded_message
+                            self.reply_wait_condition.notifyAll()
+                        elif msg_id in self.__monitor_pv_handler:
+                                executor.submit(
+                                    self.process_event,
+                                    message.topic(),
+                                    msg_id,
+                                    decoded_message[msg_id]
+                                )
 
     def parse_pv_url(self, pv_url):
         protocol, pv_name = _filter_pv_uri(pv_url)
@@ -386,8 +395,9 @@ class dml:
             self.__broker.remove_topic(self.__normalize_pv_name(pv_name))
 
     def close(self):
-        self.__consume_data = False
+        # signal thread to terminate
+        if self.__thread is not None:
+            self.__consume_data = False
+            self.__thread.join()
         if self.__broker is not None:
             self.__broker.close()
-        if self.__thread is not None:
-            self.__thread.join()
\ No newline at end of file
