@@ -4,6 +4,6 @@ from k2eg.broker import Broker
    
 def test_broker():
     try:
-        Broker('test')
+        Broker('test', 'app-test')
     except ValueError:
         pytest.fail("This should not be happen")
\ No newline at end of file
